/*
The Amnet XML parser. 

Plan - Extracts raw values, passes on.

Period/Allowance - Layer between Plan and HTMLClass; makes raw data 
	presentable and passes to HTMLClass.top/bottom/bar for display.
	
HTMLclass - 5 main areas: Top-left, top-right, bottom-left, bottom-right, 
	bar-graph area.
	Allows insertion of data into (or disabling of) each of these areas, 
	with functions to then make HTML of the data.
	
barClass - Takes a percentage (optionally compared with another percentage) 
	and returns the hex color and width of the bar-chart.
*/

/*
Plan - Extracts raw values, passes on. 

	Plan.name
	Plan.period - Period
	Plan.bw.global.peak - Allowance
	Plan.bw.global.offpeak - Allowance
	Plan.bw.local.peak - Allowance
	Plan.bw.local.offpeak - Allowance
	Plan.applyPeak() - Changes the style of the peak/offpeak areas depending 
		on whether it is peak/offpeak time. Called after HTML rendered.
*/
function plan(XML) {
	// Get the text for a certain tag name
	function getElement(tagname) { return XML.getElementsByTagName(tagname).item(0).text; };
	
	this.name = getElement('RateGroupName');
	this.period = new period(getElement('Start'), getElement('Finish'));

	this.bw = { 
		global: { peak:null, offpeak: null },
		local: { peak:null, offpeak: null }
	};
	
	var Allowances = XML.selectSingleNode("//Allowances").getElementsByTagName("OctetsIn");
	this.bw.global.peak = new allowance(
		'Peak', Allowances.item(2).text, getElement('PeakOtherOctetsIn'), getElement('PeakOtherOctetsOut')
	);
	this.bw.global.offpeak = new allowance(
		'Offpeak', Allowances.item(0).text, getElement('OffpeakOtherOctetsIn'), getElement('OffpeakOtherOctetsOut')
	);
	this.bw.local.peak = new allowance(
		'Peak', Allowances.item(3).text, getElement('PeakPeerOctetsIn'), getElement('PeakPeerOctetsOut')
	);
	this.bw.local.offpeak = new allowance(
		'Offpeak', Allowances.item(1).text, getElement('OffpeakPeerOctetsIn'), getElement('OffpeakPeerOctetsOut')
	);
	
	this.applyPeak = function(){
		var currentTime = new Date();
		var hours = currentTime.getHours();
		var day = currentTime.getDay();
		var isPeak = ((hours >= 7 && hours <= 23) && day != 5 && day != 6);
		
		function setActive(active, inactive) {
			$$('.'+active).map(function(e) {e.addClassName('active');});
			$$('.'+inactive).map(function(e) {e.removeClassName('active');});
			$$('.'+active+' .activeTxt').map(function(e){e.update('Active');});
			$$('.'+inactive+' .activeTxt').map(function(e){e.update('Inactive');});
		}
		
		if( isPeak )
			setActive('Peak', 'Offpeak');
		else
			setActive('Offpeak', 'Peak');
	}
}

/*
barClass - Takes a percentage (optionally compared with another percentage) 
	and returns the hex color and width of the bar-chart
	
	barClass.periodPercent (A barClass "class/static variable", added when Period is first created)
	barClass.width
	barClass.color
*/
barClass = Class.create({
	initialize: function(percent, isComparingWithPeriod) {
		if(percent>1) percent=1;
		else if(percent<0) percent=0;
		
		this.width = Math.round(100*percent);
		
		if( !isComparingWithPeriod )
			this.color=0x688EFF; // Blue
		else
		{
			var percentIdeal=this.periodPercent/2;
			var percentCritical=percentIdeal+1/2;
			
			this.color=this.getBarColor(percentIdeal, percent, percentCritical);
		}
		
		this.color -= 0x303030;
		this.color=this.color.toString(16);
	},
	getBarColor: function(percentIdeal, percent, percentCritical)
	{
		if( percent<percentIdeal )
			return 0x44ff44; // Green; it is ideal
		else if(percent<percentCritical)
		{
			// ranges from 0 to 1; 0 if ideal, 1 if critical
			var cutbackRatio=(percent-percentIdeal)/(percentCritical-percentIdeal);
			
			// ranges from -1 to 1; -1 if ideal, 1 if critical
			cutbackRatio=(cutbackRatio-0.5)*2;
			
			// The amount of color in the active color. if 0 result will be yellow
			var sub=Math.round(0xbb*Math.abs(cutbackRatio));
			
			var r, g;
			r=g=0xff;
		
			if( cutbackRatio<0 ) r -= sub; // Smaller than 0, subtract from the red color to make green
			else g -= sub; // Greater than 0, subtract from the green to make red
			
			return (r<<16)|(g<<8)|0x44; // Combine into one color
		}
		else
			return 0xff4444; // Red; it is critical
	}
});

/*
HTMLclass - 5 main areas: Top-left, top-right, bottom-left, bottom-right, 
	bar-graph area.
	Allows insertion of data into (or disabling of) each of these areas, 
	with functions to then make HTML of the data
	
	HTMLclass.divClass
	
	HTMLclass.top.display
	HTMLclass.top.left()
	HTMLclass.top.right()
	HTMLclass.bottom.display
	HTMLclass.bottom.left()
	HTMLclass.bottom.right()
	
	HTMLclass.bar.display
	HTMLclass.bar.percent
	HTMLclass.bar.compare
	
	HTMLclass.showLayer() 
	HTMLclass.showBar()
	HTMLclass.show()
*/
HTMLclass = Class.create({
	initialize: function() { },
	showLayer: function(layer) {
		return '<div>'+
			'<div class="leftText">'+layer.left()+'</div>'+
			'<div class="rightText">'+layer.right()+'</div>'+
			'</div>';
	},
	showBar: function() {
		var bar = new barClass(this.bar.percent, this.bar.compare);
		
		return '<div>'+
			'<table><tr>'+
			'<td class="bar blankBar" style="width:'+bar.width+'%;"> </td>'+
			'<td class="bar" style="width:auto;background-color:#'+bar.color+' !important;"> </td>'+
			'</tr></table>'+
			'</div>';
	},
	show: function() {
		buf='';
		if( this.top.display)
			buf = buf+this.showLayer(this.top);
		
		if( this.bottom.display )
			buf = buf+this.showLayer(this.bottom);
		
		if( this.bar.display )
			buf = buf+this.showBar();
					
		return '<div class="'+this.divClass+'">'+buf+'</div>';
	}
});

/*
Period/Allowance - Layer between Plan and HTMLClass; makes raw data 
	presentable and passes to HTMLClass.top/bottom/bar for display.
	HTMLclass is extended for each type to display their specific types 
	of data, using a show() function to return the data as HTML when required.
	
	First part: Take raw data and make friendlier
	Second part: Extend HTMLclass to contain friendlier data, ready for display
	
	Period.HTML - HTMLClass
	Allowance.HTML - HTMLClass
*/
function allowance(name, bytesAllowed, bytesDownloaded, bytesUploaded) {
	var percent = (bytesDownloaded/bytesAllowed);
	var bytesDownPerByteUp = (bytesDownloaded+1)/(bytesUploaded+1); // +1 to both has miniscule impact, but protects from divide by 0 errors, here and below
	
	var quotaGB = Math.round(bytesAllowed/Math.pow(10,9));
	var usedGB = (percent*quotaGB).toFixed(1);
	
	var myHTMLclass=Class.create(HTMLclass,{
		divClass: 'quota '+name,
		top: {
			display: true,
			left: function() {
				return '<span class="name">'+name+' - '+quotaGB+'GB</span>';
			},
			right: function() {
				return (quotaGB-usedGB).toFixed(1)+'/'+quotaGB+'GB left';
			}
		},
		bottom: {
			display: true,
			left: function() {
				return '<span class="activeTxt"></span>';
			},
			right: function() {
				var downGBupGB=Math.round(usedGB)+':'+Math.round(usedGB / bytesDownPerByteUp)+'GB down:up';
				var ratio = parseFloat(bytesDownPerByteUp).toFixed(1)+':1';
				return downGBupGB+' ('+ratio+')';
			}
		},
		bar: {
			display: true,
			percent: percent,
			compare: true
		}
	});
	this.HTML = new myHTMLclass();
}

function period(startDateText, resetDateText)
{
	var periodDays;
	var periodPercent;
	
	(function() {
		var startDate=new Date(startDateText.substring(0,4), startDateText.substring(5,7)-1, startDateText.substring(8,10));
		var resetDate=new Date(resetDateText.substring(0,4), resetDateText.substring(5,7)-1, resetDateText.substring(8,10));
		var currentTime = new Date();
		
		var dateRemaining = resetDate - currentTime;
		var msRemaining = new Date(dateRemaining).getTime()
		var dateWholePeriod = resetDate - startDate;
		var msWholePeriod = new Date(dateWholePeriod).getTime();
	
		periodDays = Math.floor(msWholePeriod/(1000*60*60*24));
		periodPercent = 1.0-msRemaining/msWholePeriod;
	})();
	
	barClass = Class.create(barClass, { periodPercent: periodPercent });
	
	var myHTMLclass=Class.create(HTMLclass,{
		divClass: 'time',
		top: {
			display: true,
			left: function() { return '<span class="name">Quota Resets:</span>'; },
			right: function() {
				var days = Math.floor(periodDays*(1.0-periodPercent));
				var hours = Math.floor(periodDays*(1.0-(periodPercent+(days/periodDays)))*24);
				var minutes = Math.floor(periodDays*(1.0-(periodPercent+((days+hours/24)/periodDays)))*24*60);
					
				if( days < 1 )
					return hours+' hours, '+minutes+' minutes';
				else
					return days+' days, '+hours+' hours';
			}
		},
		bottom: { display: false },
		bar: {
		display: true,
			percent: periodPercent,
			compare: false
		}
	});
	this.HTML = new myHTMLclass();
}
